/*
 * @(#)inquirer.java	1.0 97/04/18
 * 
 * Copyright (c) 1997 Leonardo Boselli. All Rights Reserved.
 *
 * Permission to use, copy, and distribute the software and its documentation
 * for NON-COMMERCIAL purposes and without fee is hereby granted provided that
 * all copyright notices appear and no modification is performed.
 *
 * I MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. I SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR
 * ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE PERFORMANCE,
 * SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR
 * COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE SUPPORT MACHINES,
 * OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE SOFTWARE COULD LEAD DIRECTLY
 * TO DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE
 * ("HIGH RISK ACTIVITIES"). I SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
 * WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.
 */

import java.sql.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A simple application for <i>SQL statements</i> testing
 *
 * @version 1.0 beta - 18 April 1997
 * @author Leonardo Boselli <a href="mailto:boselli@ivg.it">boselli@ivg.it</a>
 */

public class Inquirer implements ActionListener, ItemListener {
  private String copyright =
    "Copyright (c) 1997 Leonardo Boselli - boselli@ivg.it";
  private Frame fMain;                  // the main frame
  private TextField tfDriver;           // text field for the JDBC Driver
  private TextField tfDatabase;         // text field for the database URL
  private TextField tfQuery;            // text field for the query
  private Label lStatus;                // label performing a status bar
  private Connection DBConnection;      // database connection
  private Statement SQLStatement;       // SQL statement
  private ResultSet rsQuery;            // query result set
  private List[] lQueryColumn;          // database query columns
  private Panel pResultSet;             // container for query columns
  private ScrollPane scroll;            // scroll pane for query columns
/**
 * The main entry function. It constructs an Inquirer object
 * and initializes it.
 */
  public static void main(String args[]) {
    Inquirer inquirer = new Inquirer(); // inquirer object created
    inquirer.init();                    // inquirer object initialized
  }
/**
 * Initializes private variables
 */
  public Inquirer() {
    super();
    fMain = new Frame("Inquirer");      
    WindowAdapter waMain =              // this adapter acts when the
      new WindowAdapter() {             // window is closed
        public void windowClosing(WindowEvent e) {
          closeSql();
          System.exit(0);               // exits
        }
      }
    ;
    fMain.addWindowListener(waMain);    // sets the window components...
    tfDriver = new TextField("sun.jdbc.odbc.JdbcOdbcDriver",80);
    tfDatabase = new TextField("jdbc:odbc:",80);
    FocusAdapter faDatabase =           // this adapter connects the DB
      new FocusAdapter() {              // when tfDatabase looses focus
        public void focusLost(FocusEvent e) {
          DBConnect(tfDatabase.getText());
        }
      }
    ;
    tfDatabase.addFocusListener(faDatabase);
    tfDatabase.addActionListener(this);
    tfQuery = new TextField("SELECT * FROM ",80);
    tfQuery.addActionListener(this);
    lStatus = new Label(copyright,Label.LEFT);
    pResultSet = new Panel();
    scroll = new ScrollPane();
    scroll.setSize(200,200);
  }
/**
 * Closes all the SQL objects opened.
 */
  protected void closeSql() {
    try {
      if(rsQuery != null) rsQuery.close();
      if(SQLStatement != null) SQLStatement.close();
      if(DBConnection != null) DBConnection.close();
    } catch(java.sql.SQLException exc) {
      lStatus.setText(exc.toString());
    }
  }
/**
 * Adds a component to a container using a GridBag method.
 *
 * @param p the container
 * @param c the component to be added
 * @param gbl the GridBagLayout
 * @param gbc the GridBagContainer
 */
  private void addComponent(
        Container p, Component c, GridBagLayout gbl, GridBagConstraints gbc
  ) {
    gbl.setConstraints(c,gbc);
    p.add(c);
  }
/**
 * Adds a button to a container using <i>Inquirer</i> as an ActionListener.
 *
 * @param p the container
 * @param label the button label
 */
  private void addButton(Container p, String label, Color color) {
    Button b = new Button(label);
    if(color != null) b.setForeground(color);
    b.addActionListener(this);
    p.add(b);
  }
/**
 * Initializes the components in the window.
 */
  public void init() {
    fMain.setLayout(new BorderLayout());
    Panel pTop = new Panel();
    GridBagLayout gbl = new GridBagLayout();
    pTop.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridwidth = 1;
    addComponent(pTop,new Label("Driver",Label.RIGHT),gbl,gbc);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addComponent(pTop,tfDriver,gbl,gbc);
    gbc.gridwidth = 1;
    addComponent(pTop,new Label("Database",Label.RIGHT),gbl,gbc);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addComponent(pTop,tfDatabase,gbl,gbc);
    gbc.gridwidth = 1;
    addComponent(pTop,new Label("Query",Label.RIGHT),gbl,gbc);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addComponent(pTop,tfQuery,gbl,gbc);
    Button b = new Button();
    Panel speedBar0 = new Panel();
    speedBar0.setLayout(new FlowLayout());
    addButton(speedBar0,"SELECT * FROM ",Color.red);
    addButton(speedBar0,"CREATE TABLE ",Color.red);
    addButton(speedBar0,"DROP TABLE ",Color.red);
    addButton(speedBar0," (",null);
    addButton(speedBar0,") ",null);
    addButton(speedBar0,", ",null);
    addButton(speedBar0," [",null);
    addButton(speedBar0,"] ",null);
    addButton(speedBar0,"VARCHAR",Color.blue);
    addButton(speedBar0,"INTEGER",Color.blue);
    addButton(speedBar0,"FLOAT",Color.blue);
    addComponent(pTop,speedBar0,gbl,gbc);
    Panel speedBar1 = new Panel();
    speedBar1.setLayout(new FlowLayout());
    addButton(speedBar1,"SELECT ",Color.red);
    addButton(speedBar1,"FROM ",Color.blue);
    addButton(speedBar1,"WHERE ",Color.blue);
    addButton(speedBar1,"AND ",Color.blue);
    addButton(speedBar1,"OR ",Color.blue);
    addButton(speedBar1,"INSERT INTO ",Color.red);
    addButton(speedBar1,"VALUES ",Color.blue);
    addButton(speedBar1,"UPDATE ",Color.red);
    addButton(speedBar1,"SET ",Color.blue);
    addButton(speedBar1,"DELETE FROM ",Color.red);
    addComponent(pTop,speedBar1,gbl,gbc);
    fMain.add(pTop,"North");
    fMain.add(lStatus,"South");
    scroll.add(pResultSet);
    fMain.add(scroll,"Center");
    fMain.pack();
    fMain.setVisible(true);
  }
/**
 * implements the ItemListener method. This method is used to update the rows
 * in the other columns keeping track of row examined by the user
 *
 * @param e the item event
 */
  public void itemStateChanged(ItemEvent e) {
    for(int col = 0; col < lQueryColumn.length; col++) {
      if(lQueryColumn[col].equals(e.getSource())) {
        int idx = lQueryColumn[col].getSelectedIndex();
        for(int ncol = 0; ncol < lQueryColumn.length; ncol++) {
          if(ncol != col) {
            lQueryColumn[ncol].select(idx);
          }
        }
        break;
      }
    }
  }
/**
 * connects to the Database and initializes it.
 */
  protected void DBConnect(String dbName) {
    try {
      closeSql();
// loads driver specified in the driver textfield
      Class.forName(tfDriver.getText());
// opens a connection
      DBConnection = DriverManager.getConnection(dbName);
      DatabaseMetaData DBMetadata = DBConnection.getMetaData();
// shows database metadata
      lStatus.setText(
        "Connected to "+DBMetadata.getURL()+
        " Driver "+DBMetadata.getDriverName()+
        " Version "+DBMetadata.getDriverVersion()
      );
// create a statement
      SQLStatement = DBConnection.createStatement();
    } catch(java.lang.Exception exc) {
// if an error occurs, it's displayed in the 'status bar'
      lStatus.setText(exc.toString());
    }
  }
/**
 * implements the ActionListener method. This method choose an action
 * listening to the TextFields changes.
 *
 * @param e the action event
 */
  public void actionPerformed(ActionEvent e) {
    if(e.getSource().equals(tfQuery)) {
// if the source is the query field...
      try {
        if(SQLStatement == null) {
          lStatus.setText("Database not connected");
          return;
        }
        lStatus.setText(copyright);
// executes the query and get the result set
// if the query doesn't give a result set an executeUpdate is better
// and avoids an SQLException
        rsQuery = SQLStatement.executeQuery(e.getActionCommand());
        ResultSetMetaData rsmd = rsQuery.getMetaData();
        int nCols = rsmd.getColumnCount();
// creates all the lists to store the query columns...
        lQueryColumn = new List[nCols];
        pResultSet.removeAll();
        GridBagLayout gbl = new GridBagLayout();
        pResultSet.setLayout(gbl);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
// ...first the labels (performed by buttons)...
        for(int ctCol = 1; ctCol <= nCols; ctCol++) {
          if(ctCol != nCols)
            gbc.gridwidth = 1
          ; else
            gbc.gridwidth = GridBagConstraints.REMAINDER
          ;
          Button b = new Button("["+rsmd.getColumnLabel(ctCol)+"]");
          b.addActionListener(this);
          addComponent(pResultSet,b,gbl,gbc);
        }
// ...then the lists...
        for(int ctCol = 0; ctCol < nCols; ctCol++) {
          lQueryColumn[ctCol] = new List(10,false);
          lQueryColumn[ctCol].addItemListener(this);
          if(ctCol != nCols-1)
            gbc.gridwidth = 1
          ; else
            gbc.gridwidth = GridBagConstraints.REMAINDER
          ;
          addComponent(pResultSet,lQueryColumn[ctCol],gbl,gbc);
        }
// ...finally repaints the window...
        fMain.pack();
        fMain.invalidate();
        fMain.repaint();
// ...filling the lists with the obtained data
        while(rsQuery.next()) {
          for(int ctCol = 0; ctCol < nCols; ctCol++) {
            String str = rsQuery.getString(ctCol+1);
            if(str == null) str = "-";
            lQueryColumn[ctCol].add(str);
          }
        }
      } catch(java.sql.SQLException exc) {
// if an error occurs, it's displayed in the 'status bar'
        lStatus.setText(exc.toString());
      }
    } else if(e.getSource() instanceof Button){
// if the source is a Button...
      if(((Button)e.getSource()).getForeground().equals(Color.red)) {
// ...and it's red, fill the tfQuery with its label,...
        tfQuery.setText(((Button)e.getSource()).getLabel());
      } else {
// ...otherwise append the Button label to the textfield
        tfQuery.setText(
          (tfQuery.getText())+
          (((Button)e.getSource()).getLabel())
        );
      }
      tfQuery.setSelectionStart(tfQuery.getText().length());
      tfQuery.requestFocus();
    }
  }
}
